/*
  GNUJSP - a free JSP implementation
  Copyright (C) 1998-1999, Vincent Partington <vinny@klomp.org>
  Copyright (C) 1999, Yaroslav Faybishenko <yaroslav@cs.berkeley.edu>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
package gnu.jspengine;



import java.io.*;


/** Special CL for JSP pages.
 * The original GNUJSP CL was a 1.0 class loader. That doesn't work if
 * you're using webapps so this one is a 1.2 class loader.
 *
 * @author Nic Ferrier <nferrier@tapsellferrier.co.uk>
 */
class JspClassLoader extends ClassLoader 
{

  /** Where the JSP class files are found.
   */
  File repository;


  /** Ensure the loader is created with a parent loader.
   */
  public JspClassLoader (File repository, ClassLoader parent) 
  {
    super(parent);
    this.repository = repository;
  }


  /** Find the class in the servlet classpath.
   */
  public Class findClass(String className)
    throws ClassNotFoundException
  {
    // System.out.println("JSP asked to find: " + className + ".class");
    File file = JspServlet.getFileName (repository, className, ".class", false);
    try
      {
	byte[] buf  = null;
	FileInputStream in = new FileInputStream(file);
	byte[] classData = readClassBytes(in);
	Class jspServlet=defineClass(className, classData, 0, classData.length);
	return jspServlet;
      }
    catch (Exception e)
      {
	throw new ClassNotFoundException("whoops!");
      }
  }

  /** Uses a buffer of 5Kb to read in the class.
   */
  private static byte[] readClassBytes(InputStream in)
    throws IOException
  {
    ByteArrayOutputStream out=new ByteArrayOutputStream();
    byte[] buf=new byte[5000];
    int red=in.read(buf,0,5000);
    while (red>-1)
      {
	out.write(buf,0,red);
	red=in.read(buf,0,5000);
      }
    return out.toByteArray();
  }
    
}
