package gnu.jspengine;


import java.io.*;
import java.util.*;
import org.xml.sax.AttributeList;
import org.xml.sax.HandlerBase;
import org.xml.sax.SAXException;

/**
 * 
 * Create JspNode Tree from XML Saxparser Actions.
 *
 */ 

class JspDocHandler extends HandlerBase
{
  static private boolean debug = false;

  private JspNode jspBody = null;
  private int level = 0;
  private JspNode currentElement = null;

  private void tabs(int i) 
  {
    while (i-- > 0) 
      {
	xprint("    ");
      }
  }
  private void xprint(String s) 
  {
    JspNode textNode = new JspNode(JspNode.TEMPLATE_TEXT);
    //  textNode.setPos(pos);
    textNode.setAttribute("text", s);
    jspBody.addChild(textNode);
  }
  private void xprintln(String s) 
  {
    JspNode textNode = new JspNode(JspNode.TEMPLATE_TEXT);
    //  textNode.setPos(pos);
    textNode.setAttribute("text", s+"<br>");
    jspBody.addChild(textNode);
  }
  private void print(String s) 
  {
    // 	xprint("adding text to node type "+JspNode.tagNames[currentElement.getType()]+"<br>");
    JspNode textNode = new JspNode(JspNode.TEMPLATE_TEXT, currentElement);
    //  textNode.setPos(pos);
    textNode.setAttribute("text", s);
    currentElement.addChild(textNode);
  }
  private void println(String s) 
  {
    JspNode textNode = new JspNode(JspNode.TEMPLATE_TEXT, currentElement);
    //  textNode.setPos(pos);
    textNode.setAttribute("text", s+"\n");
    currentElement.addChild(textNode);
  }
  //     private HashTable 
  public JspDocHandler() 
  {
  }
  public void startDocument() throws SAXException 
  {
  }
  public void endDocument() throws SAXException 
  {
  }
  public void startElement(String name, AttributeList atts) 
  {
    int ntype = -1;
    boolean isChild = false;
    JspNode newNode = null;
    boolean makeNewNode = true;

    if (debug) xprintln("startElement "+name);

    if ("jsp:declaration".equals(name)) 
      {
	ntype = JspNode.JSP_DECLARATION; 
      }
    else if ("jsp:directive.page".equals(name)) 
      {
	ntype = JspNode.JSP_PAGE_DIRECTIVE; 
      }
    else if ("jsp:root".equals(name)) 
      {
	ntype = 0;
	makeNewNode = false;
      }
    else if ("jsp:expression".equals(name)) 
      {
	ntype = JspNode.JSP_EXPRESSION;
	isChild = true;
      }
    else if ("jsp:scriptlet".equals(name)) 
      {
	ntype = JspNode.JSP_SCRIPTLET;
	isChild = true;
      }
    else if ("jsp:usebean".equals(name)) 
    {
      ntype = JspNode.JSP_USEBEAN;
      isChild = true;
    }
    else if ("jsp:getProperty".equals(name)) 
    {
      ntype = JspNode.JSP_GETPROPERTY;
      isChild = true;
    }
    else if ("jsp:setProperty".equals(name)) 
      {
	ntype = JspNode.JSP_SETPROPERTY;
	isChild = true;
      }
    else if ("jsp:include".equals(name)) 
      {
	ntype = JspNode.JSP_INCLUDE;
	isChild = true;
      }
    else if ("jsp:forward".equals(name)) 
      {
	ntype = JspNode.JSP_FORWARD;
	isChild = true;
      }
    if (makeNewNode) 
      {
	newNode = new JspNode((ntype < 0) ? 0 : ntype, 
			      currentElement);
	if (isChild) 
	  {
	    currentElement.addChild(newNode);
	  }
	currentElement = newNode;
      }
    if (ntype < 0) 
      {
	tabs(level);
	xprintln("start: "+name);
	for (int i = 0; i < atts.getLength(); i++) 
	  {
	    String aname = atts.getName(i);
	    String type = atts.getType(i);
	    String value = atts.getValue(i);
	    tabs(level);
	    xprintln("attr: "+aname+":"+type+"="+value);
	  }
	level++;
      }
    else if (ntype == JspNode.JSP_PAGE_DIRECTIVE) 
      {
	for (int i = 0; i < atts.getLength(); i++) 
	  {
	    String aname = atts.getName(i);
	    String type = atts.getType(i);
	    String value = atts.getValue(i);
		
	    jspBody.setAttribute(aname, value);
	  }
      }
    else if (ntype == JspNode.JSP_USEBEAN
	     || ntype == JspNode.JSP_GETPROPERTY
	     || ntype == JspNode.JSP_SETPROPERTY
	     || ntype == JspNode.JSP_FORWARD
	     || ntype == JspNode.JSP_INCLUDE
	     ) 
      {
	for (int i = 0; i < atts.getLength(); i++) 
	  {
	    String aname = atts.getName(i);
	    String type = atts.getType(i);
	    String value = atts.getValue(i);
		
	    currentElement.setAttribute(aname, value);
	  }
      }
  }

  public void characters(char[] ch,
			 int start,
			 int length)
    throws SAXException
  {
    int type = currentElement.getType();
    if (type == JspNode.JSP_DECLARATION) 
      {
	String declaration = jspBody.getAttribute("declaration");
	if (declaration != null) 
	  {
	    declaration += new String(ch, start, length);
	  }
	else 
	  {
	    declaration = new String(ch, start, length);
	  }
	jspBody.setAttribute("declaration", declaration);
      }
    else if (type == JspNode.JSP_SCRIPTLET) 
      {
	print(new String(ch, start, length));
      }
    else if (type == JspNode.JSP_EXPRESSION) 
      {
	print(new String(ch, start, length));
      }
    else if (type == JspNode.JSP_USEBEAN) 
      {
	print(new String(ch, start, length));
      }
    else 
      {
	if (debug) xprintln("--- string start (type="+type+") ---");
	int l = length;
	while (l > 30000) 
	  {
	    print(new String(ch, start, 30000));
	    start += 30000;
	    l -= 30000;
	  }
	if (l > 0) 
	  {
	    print(new String(ch, start, l));
	  }
	if (debug) xprintln("--- string end ---");
      }
  }

  public void ignorableWhitespace(char[] ch,
				  int start,
				  int length)
    throws SAXException
  {
    StringWriter w = new StringWriter();
    w.write(ch, start , length);

    tabs(level);
    if (debug) xprintln("--- ignorableWhiteSpace start ---");
    tabs(level);
    println(w.toString());
    tabs(level);
    if (debug) xprintln("--- ignorableWhiteSpace end ---");
  }
  public void endElement(String name) 
  {
    if (debug) xprintln("endElement "+name);

    level--;
    currentElement = currentElement.getParent();
  }
  public void setJspBody(JspNode jb) 
  {
    jspBody = jb;
    currentElement = jb;
  }
}
