/*
  $Id: StringQuotesTokenizer.java,v 1.1.1.1 2002/04/23 12:47:47 nferrier Exp $ 

  GNUJSP - a free JSP implementation
  Copyright (C) 1999, Gavin Porter <gavinporter@europe.com>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, =
  USA.
*/

package gnu.jspengine;


// This class is similar to StringTokenizer. It uses the default
// StringTokenizer delimiters, i.e. whitespace,
// but allows a quoted string, that contains spaces, to be passed as a
// single token.
// The quotes are always removed from the token

import java.util.StringTokenizer;
import java.util.NoSuchElementException;

public class StringQuotesTokenizer
{
  private String str; //  holds a local copy of original string

  //  standard delimiter as used by StringTokenizer 
  private static String spaceDelimiter=" \t\n\r\f"; 
  //  our version used when searching for an end quote
  private static String quoteDelimiter="\"\t\n\r\f"; 

  private StringTokenizer tok;

  public StringQuotesTokenizer(String str) 
  {
    tok=new StringTokenizer(str, spaceDelimiter, false);
    this.str=str; 
  }

  public boolean hasMoreTokens() 
  {
    return tok.hasMoreTokens();
  }

  public String nextToken() 
  {
    //  always reset delimiter to include space
    String nt=tok.nextToken(spaceDelimiter); 

    //  no quotes so just return the token
    if (nt.charAt(0)!='"') return nt; 

    //  found a delimiter at the start of the token
    if (nt.charAt(nt.length()-1)=='"') 
      {
	//  found delimiter at end so strip out quotes and return token
	return nt.substring(1,nt.length()-1);
      } else 
	{
	  //  search for rest of token then strip out quotes
	  //  Note: we can't use hasMoreTokens to check for 
	  //  their existence because we're changing delimiters
	  try 
	    {
	      nt=nt.substring(1)+tok.nextToken(quoteDelimiter);
	    } catch (NoSuchElementException e1) 
	      {
		//  next token doesn't exist so just return nt without the quotes
		return nt.substring(1);
	      } try 
		{
		  //  if we're not at the end of the String then we
		  //  have a spare end quote left in the string,
		  //  so retrieve this and discard 
		  tok.nextToken(spaceDelimiter);
		} catch (NoSuchElementException e2) { }
	  return nt;					
	}
  }

  public int countTokens() 
  {
    int count = 0;
    int currpos = 0;
    int maxPosition=str.length();
    boolean retTokens=false;
    String delimiters=spaceDelimiter;

    while (currpos < maxPosition) {	
      while ((currpos < maxPosition) 
	     && (delimiters.indexOf(str.charAt(currpos)) >= 0)) 
	{
	  currpos++;
	}

      if (currpos >= maxPosition) 
	{
	  break;
	}

      if (str.charAt(currpos)=='"') 
	{
	  //  we've found a quoted token
	  delimiters=quoteDelimiter;
	  currpos++;
	  while ((currpos < maxPosition) 
		 && (delimiters.indexOf(str.charAt(currpos)) < 0)) 
	    {
	      currpos++;
	    }
	  while ((currpos < maxPosition) 
		 && (delimiters.indexOf(str.charAt(currpos)) >= 0)) 
	    {
	      currpos++;
	    }
	  delimiters=spaceDelimiter;
	} else { 
	  while ((currpos < maxPosition) 
		 && (delimiters.indexOf(str.charAt(currpos)) < 0)) 
	    {
	      currpos++;
	    }
	}
      count++;
    }
    return count;
  }
}
