/*
  $Id: JspMsgBundle.java,v 1.1.1.1 2002/04/23 12:47:48 nferrier Exp $ 

  GNUJSP - a free JSP implementation
  Copyright (C) 2000 - Mark Wielaard <mark@klomp.org>

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
  USA.
*/

package gnu.jspengine.prop;

import java.util.ListResourceBundle;

/**
 * Fallback resource bundle that contains all the (English) strings that GNUJSP
 * uses. It is used when JspConfig cannot load the JspMsgRes properties files.
 * It makes GNUJSP more robust when loading the localised strings goes wrong
 * (which isn't completely unlikely since getting loading resource and bundles
 * right with all these dynamic and nested classloaders is not trivial).
 * <P>
 * ApacheJserv 1.1 for example has a bug in AdaptiveClassLoader that prevents
 * normal resource bundles to be loaded automatically.
 * <P>
 * This class makes it easier to see why something goes wrong when making a
 * mistake with the config files since now GNUJSP can at least find its own
 * error messages.
 * <P>
 * WARNING - It contains the exact same messages as JspMsgRes.properties.
 * If you change any string in this class you must also change it in the
 * JspMsgRes.properties file and ask people to add the translation to the
 * localised JspMsgRes_XX.properties files.
 *
 * @author Mark Wielaard (mark@klomp.org)
 */
public class JspMsgBundle extends ListResourceBundle {

    static final Object[][] contents = {
        {"M000","No error!"},
        {"M001","gnujsp: missing init parameter "},
        {"M002","unexpected EOF while parsing attributes"},
        {"M003","Encountered unnamed JSP directive."},
        {"M004","GNUJSP internal error: You must set pagebase to use translation-time includes with this servlet engine"},
        {"m005","GNUJSP internal error: You may try to set pagebase to use translation-time includes with this 2.1 servlet engine, please tell gnujsp@gjt.org"},
        {"M006","GNUJSP: internal error: realPath == null!"},
        {"M007","GNUJSP: Currently the taglib directive is not supported."},
        {"M008","GNUJSP: Unrecognized JSP directive."},
        {"M009","GNUJSP: Badly formatted jsp tag"},
        {"M010","GNUJSP: The jsp:plugin tag is not supported."},
        {"M011","GNUJSP: Unrecognized JSP tag."},
        {"M012","GNUJSP: closing tag does not match open tag"},
        {"M013","Check if you forgot a"},
        {"M014","GNUJSP: Cannot interpret pathInfo, please report configuration to gnujsp@gjt.org"},
        {"M015","see server log for details"},
        {"M016","You might want to consider creating an error page."},
        {"M017","Error processing JSP page "},
        {"M018","Exception caught. Report to Administrator. Details sent to server log."},
        {"M019","GNUJSP: Please report to Administrator. No JSP error page registered."},
        {"M020","GNUJSP: missing closing tag"},
        {"M021","GNUJSP WARNING: Using workaround: servlet engine implementation of ServletContext.getResource(path) is broken."},
        {"M022","GNUJSP: could not create"},
        {"M023","Error compiling source "},
        {"M024","GNUJSP: unhandled exception"},
        {"M025","Exception while servicing request for"},
        {"M026","GNUJSP: internal error "},
        {"M027","GNUJSP: internal error, could not find resourceImpl"},
        {"M028","GNUJSP WARNING: it is unsafe to execute SingleThreadModel servlets with GNUJSP; see docs"},
        {"M029","GNUJSP: could not re-construct URL!"},
        {"M030","GNUJSP: illegal code line format"},
        {"M031","addCookie not allowed for wrapped request"},
        {"M032","GNUJSP: syntax error in JSP tag. No quote after name"},
        {"M033","GNUJSP: illegal parameter tag"},
        {"M034","GNUJSP: file/directory not writable "},
        {"M198","JspWriter: buffer size exceeded"},
        {"M199","JspWriter: illegal clear when some data was written."},
        {"M200","Page directive: invalid value for isThreadSafe"},
        {"M201","Page directive: illegal value for attribute 'isErrorPage'"},
        {"M202","Page directive: Buffer size must be of format Nkb."},
        {"M203","Page directive: buffer=\"none\" and autoFlush=\"false\" not allowed"},
        {"M204","% not followed by >, as expected."},
        {"M205","/ not followed by >, as expected."},
        {"M206","Page directive: multiple attribute definition detected: contentType"},
        {"M207","Page directive: Invalid attribute"},
        {"M208","Unable to forward exception to error page"},
        {"M209","Page directive: No mapping for language"},
        {"M210","Include directive: must have a \"file\" attribute."},
        {"M211","Include directive: must have a relative \"file\" attribute."},
        {"M220","runtime attribute: multiple expressions and mixing of expressions and string constants not permitted"},
        {"M221","jsp:useBean: duplicated id found"},
        {"M229","jsp:useBean: could not instantiate bean"},
        {"M230","jsp:useBean: you must specify the id of the bean"},
        {"M231","jsp:useBean: Missing 'type' or 'class' attribute"},
        {"M232","jsp:useBean: cannot specify session if session=false"},
        {"M233","jsp:useBean: you cannot specify both a class and a beanName"},
        {"M234","jsp:useBean: could not instantiate bean (class not found)"},
        {"M235","jsp:setProperty: missing 'name' attribute"},
        {"M236","jsp:setProperty: missing 'property' attribute"},
        {"M237","jsp:setProperty: illegal combination of 'param' and 'value'"},
        {"M238","jsp:setProperty: couldn't find write method for property"},
        {"M239","jsp:setProperty: property not found"},
        {"M240","jsp:setProperty: conversion from string not possible for attribute"},
        {"M241","jsp:setProperty: using undefined bean instance"},
        {"M245","jsp:getProperty: missing 'name' attribute"},
        {"M246","jsp:getProperty: missing 'property' attribute"},
        {"M247","jsp:getProperty: bean is null"},
        {"M248","jsp:getProperty: property not found"},
        {"M250","jsp:include: missing 'page' attribute"},
        {"M251","jsp:include: missing 'flush' attribute"},
        {"M252","jsp:include: attribute 'flush' must be 'true'"},
        {"M253","jsp:include: file not found"},
        {"M255","jsp:forward: missing 'page' attribute"},
        {"M256","jsp:forward: file not found"},
        {"M280","Illegal value for jsp_precompile parameter"},
        {"M330","Already called getWriter"},
        {"M331","Already called getOutputStream"},
        {"M340","unrecognized scope"},
        {"M490","Request dispatcher not found."},
        {"M999","Resources available only in English."}
    };

    public Object[][] getContents() {
        return contents;
    }
}
